#########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""BaseEntry: base class for bibliographic references;
defines the common schema elements and provides some
basic functionality """

from DateTime import DateTime
from Acquisition import aq_base
from ComputedAttribute import ComputedAttribute

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import BaseContent, DisplayList

from Products.CMFBibliographyAT.interfaces import IBibliographyExport

from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema

BaseEntrySchema = HeaderSchema + \
                  AuthorSchema + \
                  CoreSchema + \
                  TrailingSchema

class BaseEntry(BaseContent):
    """Base content for bibliographical references content types
    """

    global_allow = 0
    content_icon = 'ref_icon.png'

    schema = BaseEntrySchema

    __implements__ = (BaseContent.__implements__,
                      IBibliographyExport,
                     )

    actions = (
        {'id'           : 'view',
         'name'         : 'View',
         'action'       : 'string:${object_url}/bibliography_entry_view',
         'permissions'  : (CMFCorePermissions.View,),
         'category'     : 'object',
         },
        )

    # the default source

    def Source(self):
        """
        the default source format
        """
        try:
            source = self.DefaultSource()
        except AttributeError:   # don't blow if we have no skin context
            source = self.portal_type
        return source


    # enable field sharing between 'abstract' and 'description'

    def getAbstract(self, **kw):
        """ get the 'description' and put it in the 'abstract'
        """
        return self.Description()

    def setAbstract(self, val, **kw):
        """ synchronize 'abstract' and 'description'
        """
        self.setDescription(val)

    # helper method for direct attribute access
    # !! Should not be called anymore since Archetypes
    # !! builds automatic getFieldName() methods
    # rr: still needed by the bibtex renderer

    def getFieldValue(self, field_name):
        """
        get a field's value
        """
        field = self.getField(field_name)
        value = getattr(self, field.accessor)()
        if value:
            return value
        else:
            try:
                return field.getDefault()
            except TypeError:
                # AT1.3 compliant
                return field.getDefault(self)

    # custom methods for author handling

    def getAuthorList(self):
        """
        returns the list of author dictionaries for editing
        assumes attribute storage for authors
        """
        if hasattr(self, 'publication_authors'):
            return self.publication_authors
        else:
            return []

    def setAuthors(self, authors=[], nextauthor={}):
        """
        authors mutator
        stores authors as attribute right away
        """
        self.publication_authors = authors
        if nextauthor:
            # all of a sudden the last author was sometimes
            # saved twice and I have no idea why
            known_authors = self.publication_authors
            if known_authors and not self._cmp_dict(known_authors[-1],
                                                    nextauthor):
                self.publication_authors.append(nextauthor)
            if not known_authors:
                self.publication_authors.append(nextauthor)

    def _cmp_dict(self, d1, d2):
        """utility method to compare dictionaries"""
        k1 = d1.keys()
        k2 = d2.keys()
        k1.sort()
        k2.sort()
        if k1 != k2: return 0
        for k in k1:
            if d1[k] != d2[k]: return 0
        return 1

    def Authors(self, sep=', ', lastsep=', and ',
                abbrev=1, lastnamefirst=1, withURL=0):
        """
        string formated author list (maybe this should go in the skin?).
        assumes attribute storage for authors
        
        Arguments:

        - sep: specifies the separator between authors
               (exept for the last)
               default: ', '
                    
        - lastsep: the last separator;
                    default: ', and ' (the ',' is removed if there
                    are only two authors)
                    
        - abbrev: if true, first and middle names are abbreviated
                    default: 1

        - lastnamefirst: if true, the ordering is
                    lastname, firstname, middlename
                    otherwise it is firstname, middlename, lastname
                    default: 1

        - withURL: whether or not to embed the authorname in an
                    anchor tag pointing to his homepage (if present)
                    default: 0
        """
        if not hasattr(self, 'publication_authors') and not hasattr(self, 'member_publication_authors_cache'):
            return "No authors specified"

        authors = tuple(getattr(self, 'publication_authors', [])) + tuple(getattr(self, 'member_publication_authors_cache', []))

        nofa = len(authors)

        if nofa == 0:
            return "No authors specified"

        if nofa == 1:
            return self._formatAuthor(authors[0],abbrev,
                                      lastnamefirst,withURL)

        if nofa == 2:
            if ',' in lastsep and not abbrev:
                lastsep = lastsep.replace(',','')

            a1 = self._formatAuthor(authors[0],abbrev,
                                    lastnamefirst,withURL)
            a2 = self._formatAuthor(authors[1],abbrev,
                                    lastnamefirst,withURL)

            return a1 + lastsep + a2

        result = ""
        for author in authors[:-2]:
            result += self._formatAuthor(author,abbrev,
                                         lastnamefirst,withURL)\
                      + sep
        result += self._formatAuthor(authors[-2],abbrev,
                                     lastnamefirst,withURL)\
                  + lastsep
        result += self._formatAuthor(authors[-1],abbrev,
                                     lastnamefirst,withURL)

        return result

    def _formatAuthor(self,author,abbrev,lastnamefirst,withURL):
        """
        string formats an individual author
        """
        fname = author.get('firstname', '')
        mname = author.get('middlename', '')
        lname = author.get('lastname', '')
        # homepage = author.get('homepage', '')
        astring = ''

        if abbrev:
            if fname: fname = fname[0]
            if mname: mname = mname[0]

        if lastnamefirst:
            if not fname and not mname:
                astring = "%s" % lname
            elif abbrev:
                astring = "%s, %s%s" % (lname, fname, mname)
            else:
                astring = "%s, %s %s" % (lname, fname, mname)
                # remove the trailing blank if mname is empty
                if not mname: astring = astring.strip()
        else:
            if abbrev:
                astring = "%s%s %s" % (fname, mname, lname)
            else:
                astring = "%s %s %s" % (fname, mname, lname)
                # remove double blanks (happens if mname is empty)
                astring = astring.replace('  ', ' ')

        if withURL:
            homepage = self.getAuthorURL(author)
            if homepage:
                return """<a href="%s">%s</a>""" % (homepage, astring)
            else:
                return astring
        else:
            return astring

    def getAuthorURL(self, author):
        """
        finds the URL of the author's hompage
        First, looks if 'homepage' is set in the 'author' dictionary
        Second, asks the tool for an entry for 'firstname' + 'lastname'
        Third, asks the membership tool whether there is a user 'lastname'
        """
        # 1. on object base
        homepage = author.get('homepage', None)
        if homepage: return homepage

        # 2. on bibfolder base
        parent = self.aq_parent
        full_name = author.get('firstname', '') \
                    + ' ' \
                    + author.get('lastname', '')

        if parent.meta_type == 'BibliographyFolder':
            homepage = parent.AuthorURLs().get(full_name.strip(), None)
        if homepage: return homepage

        # 3. Site wide look-up by fullname
        membertool = getToolByName(self, 'portal_membership', None)
        full_names = [m.getProperty('fullname', 'dummy') \
                      for m in membertool.listMembers()]
        try:
            index = full_names.index(full_name)
            user = membertool.listMembers()[index].getId()
            return membertool.getHomeUrl(user, 1)
        except ValueError:
            pass

        # 4. Site wide look-up by first initial plus lastname

        fname = author.get('firstname', '')
        lname = author.get('lastname', '')
        abbrev_name = ''
        if fname: abbrev_name += fname + ' '
        abbrev_name += lname
        abbrev_name = abbrev_name.strip()

        abbrev_names = []
        for name in full_names:
            tokens = name.split()
            if len(tokens) > 1:
                aname = tokens[0][0] + ' ' + tokens[-1]
            else:
                aname = name
            abbrev_names.append(aname)
        try:
            index = abbrev_names.index(abbrev_name)
            user = membertool.listMembers()[index].getId()
            return membertool.getHomeUrl(user, 1)
        except ValueError:
            pass


        # 5. admittedly stupid default
        user = author.get('lastname', '').lower()
        homepage = membertool.getHomeUrl(user, 1)

        return homepage

    def getURL(self, defaultURL=None):
        """
        the publication_url if set, otherwise a link to PubMed
        if pmid is set or the default otherwise
        """
        url = self.getPublication_url()
        if url:
            return url
        else: return defaultURL

    def PMID(self):
        """
        to be available for all types
        overwritten by article
        """
        return None

    def ISBN(self):
        """
        to be available for all types
        overwritten by books
        """
        return None

    def pre_validate(self, REQUEST, errors):
        if REQUEST.get('more_authors',None):
            self.setAuthors(REQUEST.get('authors',[]),
                            REQUEST.get('nextauthor',None))
            if REQUEST.form.get('nextauthor',None):
                del REQUEST.form['nextauthor']
            errors['publication_authors']='Next author enabled'
        else:
            self.setAuthors(REQUEST.get('authors',[]),
                            REQUEST.get('nextauthor',None))

    def getPublicationDate(self):
        """
        Returns the publication date as DateTime
        or None if it is not well defined
        """
        year = self.getField('publication_year').get(self)
        try:
            year = int(year)
        except ValueError:
            year = None

        month = self.getField('publication_month').get(self)
        if month:
            try:
                month = int(month)
            except ValueError:
                try:
                    # This is probably a string
                    monthcomp = month.split(' ')
                    month = 1
                    for m in monthcomp:
                        if m.lower() in DateTime._monthmap.keys():
                            month = m
                            continue
                except (ValueError, AttributeError, IndexError):
                    month = 1
        else:
            month = 1

        return year and DateTime('%s/%s/01' % (year, month)) or None

    publication_date = ComputedAttribute(getPublicationDate, 1)

    def getMembers(self):
        """
        For use when members are authors, return a DisplayList of members
        """
        value = []
        if self.showMemberAuthors():
            md = getToolByName(self, 'portal_memberdata')
            membertypes = md.getAllowedMemberTypes()
            catalog = getToolByName(self, 'portal_catalog')
            results = catalog(portal_type=membertypes)
            value = [(getattr(aq_base(r), 'UID', r.getObject().UID()), (r.Title or r.id)) for r in results]
        value.insert(0, ('', '<no reference>'))
        return DisplayList(value)

    def showMemberAuthors(self):
        quickinstaller = getToolByName(self, 'portal_quickinstaller')
        return quickinstaller.isProductInstalled('CMFMember')

    def setMemberPublicationAuthors(self, value, **kw):
        """Default Mutator."""
        if kw.has_key('schema'):
            schema = kw['schema']
        else:
            schema = self.Schema()
            kw['schema'] = schema
        schema['member_publication_authors'].set(self, value, **kw)
        self.updateMemberPublicationAuthors()

    def updateMemberPublicationAuthors(self):
        authors = self.Schema()['member_publication_authors'].get(self, aslist=1)
        mpa = []
        for author in authors:
            data = self.getAuthorDataFromMember(author)
            if data is not None:
                mpa.append(data)
        self.member_publication_authors_cache = mpa


    # support inline citations
    def citationLabel(self, join_two='and', et_al='et al.'):
        """
        a short label for inline citations like
        "Ritz, 1995" or "Ritz and Herz, 2004" or
        "Ritz et al., 2005"
        """
        authors = self.getAuthorList()
        nofa = len(authors)
        year = self.getField('publication_year').get(self)

        if nofa == 0:
            return "Anonymous, %s" % year

        if nofa == 1:
            return "%s, %s" % (authors[0].get('lastname', ''),
                               year)        
        if nofa == 2:
            return "%s %s %s, %s" % (authors[0].get('lastname', ''),
                                     join_two,
                                     authors[1].get('lastname', ''),
                                     year)
        return "%s %s, %s" % (authors[0].get('lastname', ''),
                              et_al,
                              year) 
